;; cgi.lsp  
;;
;; v 1.0
;; v 1.1 - 'url-translate' now with new 'replace' with back refs
;; v 1.2 - put everything in context CGI for better isolation
;; v 1.3 - changed 'name' to var
;; v 1.4 - changed deprecated 'substring' to 'slice'
;; v 1.5 - parameters returned parameter name when empty, now ""
;;         set-cookkie now documented in header
;;         fix for (getenv "QUERY_STRING") on IIS
;; v 1.6 - replaced deprecated getenv and putenv with env
;; v 1.7 - dead code cleanup
;; v 1.8 - replaced int for integer
;; v 1.9 - change for version newLISP 8.7.8: put-page/eval-string
;; v 2.0 - fix for = characters in key=value string (thanks Gordon Fischer)
;;
;; define basic CGI processing tools and get GET or POST parameters ;;;

;; include this file at the beginning of each file performing CGI processing
;; example: (load "cgi.lsp")
;;
;; After having loaded this file all parameters from either GET or POST
;; method are stored as an association list in params.
;; 
;; example for accessing:
;;
;; (CGI:get "userid")   =>  "johndoe" 
;;
;; All cookies can be accessed in an association list cookies and
;; are accessed the similar to GET and PUT parameters
;;
;; example:
;;
;; (CGI:get-cookie "login")   =>  "somebody" 
;;
;; (CGI:set-cookie "password" "secret" "asite.com" "/somedir")
;;
;; CGI:set-cookie should be called immedeately before
;; finishing the header with (print "Content-type: text/html\r\n\r\n")
;; as cookies are part of the header information
;;
;; params and cookies contain the empty list '() when no
;; parameters or cookies are present
;;
;; The function 'put-page can be used to output web pages containing
;; newLISP source embedded in <%, %> tags. Inside these tags are newLISP
;; statements printing output/HTML to the webpage.
;;
;; example:
;;
;; (CGI:put-page mypage.html)
;;
;; where mypage.html contains embedded newLISP source
;;

;; put-page takes an HTML page and processes the embedded newLISP code
;; embedded in <%, %> tags
;; 

(context 'CGI)

(define (put-page file-name , page start end)
    (set 'page (read-file file-name))
    (set 'start (find "<%" page))
    (set 'end (find "%>" page))
    (while (and start end)
        (print (slice page 0 start))
        (context MAIN)
        (eval-string (slice page (+ start 2) (- end start 2)))
        (context CGI)
        (set 'page (slice page (+ end 2)))
        (set 'start (find "<%" page))
        (set 'end (find "%>" page)))
    (print page))


;; url-translate translates + into spaces and %nn hesxdigits into 
;; characters 
;;
(define (url-translate str)
   (replace "+" str " ")
   (replace "%([0-9A-F][0-9A-F])" str (char (int (append "0x" $1))) 1))

;; get-vars returns all parameter value pairs in an association list
;; i.e.: ( ("name" "johndoe") ("password" "secret") )
;; they can than be accessed using:
;;     (assoc "name" params) => "johndoe"
;; where params is the return value from get-vars
;; 
(define (get-vars input , var value var-value)
    (set 'vars (parse input "&"))
    (dolist (elmnt vars) 
	(if (find "=" elmnt) 
	    (begin
              (set 'var (first (parse elmnt "=")))
              (set 'value ((+ (find "=" elmnt) 1) elmnt)))
	    (begin
	      (set 'var elmnt)
	      (set 'value "")))
	(push (list var (url-translate value)) var-value))
    var-value)


;; get QUERY_STRING parameters from GET method if present
;;
(set 'params (env "QUERY_STRING"))
(if (not params) (set 'params ""))
(if params
	(set 'params (get-vars params)))

;; get stdin POST method parameters if present
;;
(set 'inline (read-line))
(if inline 
	(set 'params (get-vars inline)))
 
(if (not params) (set 'params '()))

;; get cookies
;;
(if (env "HTTP_COOKIE")
    (dolist (elmnt (parse (env "HTTP_COOKIE") ";"))
	(set 'var (trim (first (parse elmnt "="))))
	(set 'value (trim (last (parse elmnt "="))))
	(push (list var value) cookies))
    (set 'cookies '()))

;; set cookies
;;
;; this function should be called immedeately before
;; writing the header with (print "Content-type: text/html\r\n\r\n")
;; as cookies are part of the header information
;;
(define (set-cookie var value domain path)
    (set 'value (string value))
    (print (format "Set-Cookie: %s=%s; domain=.%s; path=%s;\n" var value domain path)))

(define (get-cookie keystr)
	(lookup keystr cookies) )


(define (get keystr)
   (lookup keystr params))

(context 'MAIN)
;; eof ;;
