;;
;; hash.lsp - functions for making  hashes in newLISP
;;
;; this module puts a thin object oritned layer around the 'sym'
;; functions for building dictionaries for associative memory access
;; called hashes in other languages
;;
;; version 1.1 - changed HASH:new to HASH:make
;; to make correct serialization of hashes possible
;;
;; version 1.2 - changed 'symbol' function can take any string
;;               much faster than last version
;;               now uses the default function HASH as constructor
;;
;; version 1.3 - changed 'symbol' to the identical but shorter 'sym'
;; version 1.4 - one 'symbols' was wringly converetd to 'sym'
;;
;; a hash key can be any string, number or symbol, 
;; a key can contain any characters and does not have to
;; be a legal newLISP symbol, but the keys:  
;; this, put, get, key, keys, value, values and the name of the
;; hash created, can not be used and are protected.
;;
;;
;; example: 
;;           (load "hash.lsp")                  ;; load this module
;;
;;           (HASH 'MyHash)                     ;; create a hash MyHash
;;
;;           (MyHash:put "someKey" 1234)        ;; create key-value entry
;;           (MyHash:put "anything" '(1 2 3))   ;; create key-value entry
;;
;;           (MyHash:get "someKey")             => 1234
;;           (MyHash:get "anything")            => (1 2 3)
;;
;;           (MyHash:keys)                      => ("anything" "someKey")
;;
;;           (new MyHash 'MyHash-2)             => MyHash-2
;;           (MyHash-2:get "someKey")           => 1234
;;
;;           (MyHash:values)                    => ((1 2 3) 1234)

(context 'HASH)

(constant 'HASH:HASH (lambda (this) ;; use default function as constructor    
    (new 'MAIN:HASH this)))         ;; use new to copy
                                    ;; in the new context 'this' will point to it
	
(constant 'put (lambda (key value)
    (set (sym key this) value)))

(constant 'get (lambda (key)
    (eval (sym key this))))

(constant 'keys (lambda ()
    (map name (difference (symbols this)      ;; subtract illegal keys
                (append '(this put get key keys value values) 
                         (list (sym (name this) this)))))))

(constant 'values (lambda () 
    (map get (keys))))


(context MAIN)

;; eof
