;;  smtp.lsp - routines for sending mail
;;
;;  v.1.1 cleanup and comments
;;  v.1.2 better error reporting when (set 'debug-flag true)
;;  v.1.3 inserted empty line before body
;;  v.1.4 forgot to check for "\r\n" in confirm request
;;        rearranged function calls for shorter code
;;        USAGE was incomplete
;;  v.1.5 changes sys-info version ref
;;  v.1.6 added RFC821 envelope characters "<...>" to email addresses
;;

(context 'SMTP)

(set 'debug-flag nil)

;; this is the main function to use
;;
;;  USAGE:
;;
;;  ;; include the smtp module
;;  (load "/usr/share/newlisp/smtp.lsp")
;;
;;   (SMTP:send-mail "jdoe@asite.com" "somebody@isp.com" "" "Greetings"
;;                   "How are you today? - john doe -" "smtp.asite.com")
;;
;;   will send mail from address: jdoe@asite.com
;;                    to address: somebody@isp.com
;;                  subject line: Greetings
;;                  message body: Hoe are you today? - john doe-
;;                     smtp host: smtp.asite.com
;;

(define (send-mail mail-from mail-to mail-subject mail-body SMTP-server)
    (and
        (set 'from-hostname (nth 1 (parse mail-from "@")))
        (set 'socket (net-connect SMTP-server 25))
        (confirm-request "2")
        (net-send-get-result (append "HELO " from-hostname) "2")
        (net-send-get-result (append "MAIL FROM: <" mail-from ">") "2")
        (net-send-get-result (append "RCPT TO: <" mail-to ">") "2")
        (net-send-get-result "DATA" "3")
        (mail-send-header)
        (mail-send-body)
        (confirm-request "2")
        (net-send-get-result "QUIT" "2")
        (or (net-close socket) true)))

(define (confirm-request conf)
   (and
    (net-receive socket 'recvbuff 256 "\r\n")
    (if debug-flag (println recvbuff) true)
    (starts-with recvbuff conf)))
  
(define (net-send-get-result str conf)
   (set 'send-str (append str "\r\n"))
   (if debug-flag (println "sent: " send-str)) 
   (net-send socket 'send-str)
   (if conf (confirm-request conf) true))

(define (mail-send-header)
    (net-send-get-result (append "TO: " mail-to))
    (net-send-get-result (append "FROM: " mail-from))
    (net-send-get-result (append "SUBJECT: " mail-subject))
    (net-send-get-result (append "X-Mailer: newLISP v." (string (nth -2 (sys-info)))))) 

(define (mail-send-body )
    (net-send-get-result "")
    (dolist (lne (parse mail-body "\r\n")) 
        (if (= lne ".") 
            (net-send-get-result "..")
            (net-send-get-result lne)))
    (net-send-get-result "."))

(define (get-error-text)
    recvbuff)

(context 'MAIN)

;; eof
